@@ -2,11 +2,12 @@ import re
 import sys
 from functools import wraps
 from inspect import Parameter, signature
-from typing import Any, get_type_hints
+from typing import Any, Union, get_type_hints
 
 import pytest
 
 from pydantic_core import ArgsKwargs, SchemaError, SchemaValidator, ValidationError, core_schema
+from pydantic_core import core_schema as cs
 
 from ..conftest import Err, PyAndJson, plain_repr
 
@@ -737,17 +738,16 @@ def test_default_factory(py_and_json: PyAndJson, input_value, expected):
 
 def test_repr():
     v = SchemaValidator(
-        {
-            'type': 'arguments',
-            'arguments_schema': [
-                {'name': 'b', 'mode': 'positional_or_keyword', 'schema': {'type': 'int'}},
+        cs.arguments_schema(
+            arguments=[
+                {'name': 'b', 'mode': 'positional_or_keyword', 'schema': cs.int_schema()},
                 {
                     'name': 'a',
                     'mode': 'keyword_only',
-                    'schema': {'type': 'default', 'schema': {'type': 'int'}, 'default_factory': lambda: 42},
+                    'schema': cs.with_default_schema(schema=cs.int_schema(), default_factory=lambda: 42),
                 },
-            ],
-        }
+            ]
+        )
     )
     assert 'positional_params_count:1,' in plain_repr(v)
 
@@ -755,20 +755,26 @@ def test_repr():
 def test_build_non_default_follows():
     with pytest.raises(SchemaError, match="Non-default argument 'b' follows default argument"):
         SchemaValidator(
-            {
-                'type': 'arguments',
-                'arguments_schema': [
+            schema=cs.arguments_schema(
+                arguments=[
                     {
                         'name': 'a',
                         'mode': 'positional_or_keyword',
-                        'schema': {'type': 'default', 'schema': {'type': 'int'}, 'default_factory': lambda: 42},
+                        'schema': cs.with_default_schema(schema=cs.int_schema(), default_factory=lambda: 42),
                     },
-                    {'name': 'b', 'mode': 'positional_or_keyword', 'schema': {'type': 'int'}},
-                ],
-            }
+                    {'name': 'b', 'mode': 'positional_or_keyword', 'schema': cs.int_schema()},
+                ]
+            )
         )
 
 
+def test_build_missing_var_kwargs():
+    with pytest.raises(
+        SchemaError, match="`var_kwargs_schema` must be specified when `var_kwargs_mode` is `'unpacked-typed-dict'`"
+    ):
+        SchemaValidator(cs.arguments_schema(arguments=[], var_kwargs_mode='unpacked-typed-dict'))
+
+
 @pytest.mark.parametrize(
     'input_value,expected',
     [
@@ -778,7 +784,7 @@ def test_build_non_default_follows():
     ],
     ids=repr,
 )
-def test_kwargs(py_and_json: PyAndJson, input_value, expected):
+def test_kwargs_uniform(py_and_json: PyAndJson, input_value, expected):
     v = py_and_json(
         {
             'type': 'arguments',
@@ -796,6 +802,48 @@ def test_kwargs(py_and_json: PyAndJson, input_value, expected):
         assert v.validate_test(input_value) == expected
 
 
+@pytest.mark.parametrize(
+    'input_value,expected',
+    [
+        [ArgsKwargs((), {'x': 1}), ((), {'x': 1})],
+        [ArgsKwargs((), {'x': 1.0}), Err('x\n  Input should be a valid integer [type=int_type,')],
+        [ArgsKwargs((), {'x': 1, 'z': 'str'}), ((), {'x': 1, 'y': 'str'})],
+        [ArgsKwargs((), {'x': 1, 'y': 'str'}), Err('y\n  Extra inputs are not permitted [type=extra_forbidden,')],
+    ],
+)
+def test_kwargs_typed_dict(py_and_json: PyAndJson, input_value, expected):
+    v = py_and_json(
+        {
+            'type': 'arguments',
+            'arguments_schema': [],
+            'var_kwargs_mode': 'unpacked-typed-dict',
+            'var_kwargs_schema': {
+                'type': 'typed-dict',
+                'fields': {
+                    'x': {
+                        'type': 'typed-dict-field',
+                        'schema': {'type': 'int', 'strict': True},
+                        'required': True,
+                    },
+                    'y': {
+                        'type': 'typed-dict-field',
+                        'schema': {'type': 'str'},
+                        'required': False,
+                        'validation_alias': 'z',
+                    },
+                },
+                'config': {'extra_fields_behavior': 'forbid'},
+            },
+        }
+    )
+
+    if isinstance(expected, Err):
+        with pytest.raises(ValidationError, match=re.escape(expected.message)):
+            v.validate_test(input_value)
+    else:
+        assert v.validate_test(input_value) == expected
+
+
 @pytest.mark.parametrize(
     'input_value,expected',
     [
@@ -835,14 +883,14 @@ def test_alias(py_and_json: PyAndJson, input_value, expected):
     ],
     ids=repr,
 )
-def test_alias_populate_by_name(py_and_json: PyAndJson, input_value, expected):
+def test_alias_validate_by_name(py_and_json: PyAndJson, input_value, expected):
     v = py_and_json(
         {
             'type': 'arguments',
             'arguments_schema': [
                 {'name': 'a', 'mode': 'positional_or_keyword', 'schema': {'type': 'int'}, 'alias': 'Foo'}
             ],
-            'populate_by_name': True,
+            'validate_by_name': True,
         }
     )
     if isinstance(expected, Err):
@@ -852,6 +900,34 @@ def test_alias_populate_by_name(py_and_json: PyAndJson, input_value, expected):
         assert v.validate_test(input_value) == expected
 
 
+def test_only_validate_by_name(py_and_json) -> None:
+    schema = core_schema.arguments_schema(
+        [
+            core_schema.arguments_parameter(name='a', schema=core_schema.str_schema(), alias='FieldA'),
+        ],
+        validate_by_name=True,
+        validate_by_alias=False,
+    )
+    v = py_and_json(schema)
+    assert v.validate_test(ArgsKwargs((), {'a': 'hello'})) == ((), {'a': 'hello'})
+    with pytest.raises(ValidationError, match=r'a\n +Missing required argument \[type=missing_argument,'):
+        assert v.validate_test(ArgsKwargs((), {'FieldA': 'hello'}))
+
+
+def test_only_allow_alias(py_and_json) -> None:
+    schema = core_schema.arguments_schema(
+        [
+            core_schema.arguments_parameter(name='a', schema=core_schema.str_schema(), alias='FieldA'),
+        ],
+        validate_by_name=False,
+        validate_by_alias=True,
+    )
+    v = py_and_json(schema)
+    assert v.validate_test(ArgsKwargs((), {'FieldA': 'hello'})) == ((), {'a': 'hello'})
+    with pytest.raises(ValidationError, match=r'FieldA\n +Missing required argument \[type=missing_argument,'):
+        assert v.validate_test(ArgsKwargs((), {'a': 'hello'}))
+
+
 def validate(config=None):
     def decorator(function):
         parameters = signature(function).parameters
@@ -1049,21 +1125,15 @@ def test_function_args_kwargs():
 def test_invalid_schema():
     with pytest.raises(SchemaError, match="'default' and 'default_factory' cannot be used together"):
         SchemaValidator(
-            {
-                'type': 'arguments',
-                'arguments_schema': [
+            schema=cs.arguments_schema(
+                arguments=[
                     {
                         'name': 'a',
                         'mode': 'positional_or_keyword',
-                        'schema': {
-                            'type': 'default',
-                            'schema': {'type': 'int'},
-                            'default': 1,
-                            'default_factory': lambda: 2,
-                        },
+                        'schema': cs.with_default_schema(schema=cs.int_schema(), default=1, default_factory=lambda: 2),
                     }
-                ],
-            }
+                ]
+            )
         )
 
 
@@ -1103,3 +1173,50 @@ def test_error_display(pydantic_version):
         '[{"type":"missing_argument","loc":["b"],"msg":"Missing required argument",'
         '"input":"ArgsKwargs((), {\'a\': 1})"}]'
     )
+
+
+@pytest.mark.parametrize('config_by_alias', [None, True, False])
+@pytest.mark.parametrize('config_by_name', [None, True, False])
+@pytest.mark.parametrize('runtime_by_alias', [None, True, False])
+@pytest.mark.parametrize('runtime_by_name', [None, True, False])
+def test_by_alias_and_name_config_interaction(
+    config_by_alias: Union[bool, None],
+    config_by_name: Union[bool, None],
+    runtime_by_alias: Union[bool, None],
+    runtime_by_name: Union[bool, None],
+) -> None:
+    """This test reflects the priority that applies for config vs runtime validation alias configuration.
+
+    Runtime values take precedence over config values, when set.
+    By default, by_alias is True and by_name is False.
+    """
+
+    if config_by_alias is False and config_by_name is False and runtime_by_alias is False and runtime_by_name is False:
+        pytest.skip("Can't have both by_alias and by_name as effectively False")
+
+    schema = core_schema.arguments_schema(
+        arguments=[
+            core_schema.arguments_parameter(name='my_field', schema=core_schema.int_schema(), alias='my_alias'),
+        ],
+        **({'validate_by_alias': config_by_alias} if config_by_alias is not None else {}),
+        **({'validate_by_name': config_by_name} if config_by_name is not None else {}),
+    )
+    s = SchemaValidator(schema)
+
+    alias_allowed = next(x for x in (runtime_by_alias, config_by_alias, True) if x is not None)
+    name_allowed = next(x for x in (runtime_by_name, config_by_name, False) if x is not None)
+
+    if alias_allowed:
+        assert s.validate_python(
+            ArgsKwargs((), {'my_alias': 1}), by_alias=runtime_by_alias, by_name=runtime_by_name
+        ) == (
+            (),
+            {'my_field': 1},
+        )
+    if name_allowed:
+        assert s.validate_python(
+            ArgsKwargs((), {'my_field': 1}), by_alias=runtime_by_alias, by_name=runtime_by_name
+        ) == (
+            (),
+            {'my_field': 1},
+        )
