@@ -17,11 +17,17 @@ from ..conftest import Err, PyAndJson
     'constraint',
     ['le', 'lt', 'ge', 'gt'],
 )
-def test_constraints_schema_validation(constraint: str) -> None:
+def test_constraints_schema_validation_error(constraint: str) -> None:
     with pytest.raises(SchemaError, match=f"'{constraint}' must be coercible to a date instance"):
         SchemaValidator(cs.date_schema(**{constraint: 'bad_value'}))
 
 
+def test_constraints_schema_validation() -> None:
+    val = SchemaValidator(cs.date_schema(gt='2020-01-01'))
+    with pytest.raises(ValidationError):
+        val.validate_python('2019-01-01')
+
+
 @pytest.mark.parametrize(
     'input_value,expected',
     [
